#use Statistics::Distributions qw(chisqrprob);

sub
percentile {
	my ($number, @values) = @_;
	my $rank = rank ($number, @values);
	return ($rank/scalar (@values));  
}

sub rank_no_sort
{
	my ($number, @sorted_values) = @_;

	        for (my $i = 0; $i < @sorted_values; $i++) {
                if ($sorted_values[$i] == $number) {
#       print "$sorted_values[$i] $number rank $i\n";
                return ($i + 1);
                }
        }
        return (-1);
}

sub rank_no_sort_fast
{
    my ($x, $a) = @_;            # search for x in array a
    my ($l, $u) = (0, @$a - 1);  # lower, upper end of search interval
    my $i;                       # index of probe
    while ($l <= $u) {
	$i = int(($l + $u)/2);
#	print($i, "\n");
	if ($a->[$i] < $x) {
	    $l = $i+1;
	}
	elsif ($a->[$i] > $x) {
	    $u = $i-1;
	} 
	else {
	    return ($i+1); # found
	}
    }
    return $i;         # return closest not found
}

sub rank {
	my ($number, @values) = @_;
	my @sorted_values = sort {$b <=> $a} @values;
	
	for (my $i = 0; $i < @sorted_values; $i++) {
		if ($sorted_values[$i] == $number) {
#	print "$sorted_values[$i] $number rank $i\n";
		return ($i + 1);
		}
	}
	return (-1);

}

sub round {
	my ($number) = shift;
	return int ($number+0.5);
}

sub
variance
{
	my ( @nums) = @_;

	my $average_deviation = 0;	
	my $variance = 0;
	my $num;
	my $skew = 0;
	my $kurtosis = 0;
	my $n = scalar (@nums);
	my $mean = mean (@nums);
	foreach $num (@nums) {
	    my $deviation = $num - $mean;
	    $average_deviation += abs($deviation);
	    $variance += $deviation**2;
	    $skew += $deviation**3;
	    $kurtosis += $deviation**4;
	}
	$average_deviation /= $n;
	$variance /= ($n - 1);
	my $standard_deviation = sqrt($variance);

	if ($variance) {
		    $skew /= ($n * $variance * $standard_deviation);
		    $kurtosis = $kurtosis/($n * $variance * $variance) - 3.0;
	}
	return ($variance, $skew, $kurtosis, $standard_deviation, $mean);
}

sub poisson_prob {
        my ($x, $m) = @_;
        my $num = (exp (-1 * $m) * power ($m, $x)) / factorial ($x);
#       print " exp " . (exp (-1 * $m)) . " power " . power ($m,$x) .
#               " factorial " . factorial ($x) . "\n";
#       print "poisoon prob $x $m $num\n";
        return ( $num) ;
}

sub cumulative_poisson {
        my ($x, $m)  = @_;
        my $total_prob = 0;
        for (my $i = 0; $i < $x; $i++) {
#               print "total prob for $x $m is $total_prob for $i\n";
                $total_prob += poisson_prob ($i, $m);
        }
#       print "TOTAL prob is $total_prob\n";
        return (1-$total_prob);
}

# binomail prob (# of trials, # of successes,prob of success)
sub binomial_prob {
        my ($n, $k, $p) = @_;
        my $prob = 1;
        $prob *= factorial($n);
        $prob /= factorial ($k);
        $prob /= factorial ($n - $k);
        $prob *= $p**$k;
        $prob *= (1 - $p)**($n-$k);
        return ($prob);
}


#calculates the cumulative probability of a binomial distribution
# Pr (Z >= k | p, n)

sub cumulative_prob {

my ($n, $k, $p) = @_;
my $cum_prob = 0;

        if ($n*$p > 50) {
                return (normal_cumulative_prob ($k, $n * $p, $n * $p * (1-$p)));
        }
        for ($j = $n; $j >= $k; $j--) {
                my $prob = 1;
                $prob *= factorial($n);
                $prob /= factorial ($j);
                $prob /= factorial ($n - $j);
                $prob *= $p **  $j;
                $prob *= (1 - $p)**($n-$j);

                $cum_prob += $prob;
        }
        return $cum_prob;
}

sub normal_cumulative_prob {
        my ($k, $mean, $var) = @_;

        my @zvalues = (1, 1.25, 1.5, 1.75, 2, 2.25, 2.5, 2.75,
                       3, 3.25, 3.5, 3.75, 4, 4.25, 4.5, 4.75,
                       5, 5.25, 5.5, 5.75, 6, 6.25, 6.5, 6.75,
                       7, 7.25, 7.5, 7.75);
        my @pvalues = (0.1586552, 0.10564, 0.06680, 0.040059, 0.0227500,
                         0.01222,
                        0.00620,  0.002979, 0.0013499, 0.000577086,
                        0.0002326, 8.84446e-05,
                         3.1686e-05,  1.06957e-05, 3.4008e-06, 1.01833e-06,
                        2.87105e-07, 7.62014e-08, 1.90364e-08, 4.47554e-09,
                        9.90122e-10, 2.06094e-10,  4.03579e-11, 7.43428e-12,
                        1.28808e-12, 2.09943e-13,  3.21965e-14,  4.66294e-15);

        if (@pvalues != @zvalues) { printf "something wrong!!! with normal\n";}
        my $stddev = $var ** (1/2);
        my $z = ($k - $mean) / $stddev;

        $i = 0;
        while ($z > $zvalues[$i]) {
                $i++;
        }

        if ($i == 0) {
                return (0.5); # not worth it
        } else {
                return ($pvalues[$i]);
        }

} # end function normal_cumulative_prob

sub power {
        my ($base, $exp) = @_;
        my $answer = 1;

        for (my $i = 1; $ i <= $exp; $i++) {
                $answer *= $base;
        }
        return $answer;
}

sub factorial {
        my ($num) = @_;
        my $answer = 1;
        if ($num == 0 || $num == 1) { return 1;}
        my $j;
        for ($j = 2; $j <= $num; $j++) {
                $answer *= $j;
        }
        return $answer;
}

sub sum {
        my (@values) = @_;
        my $sum = 0;;
        for (my $i = 0 ; $i < @values; $i++) {
                $sum += $values[$i];
        }
        return ($sum);
}

sub mean {
        my (@values) = @_;
        my $sum = 0; my $n = 0;
        for (my $i = 0; $i < @values; $i++) {
                $sum += $values[$i];
                $n++;
        }
        return ($sum/$n);
}
sub median {
        my (@values) = @_;

        my @sorted_values = sort {$a <=> $b} @values;
        my $num_items = @sorted_values;
        my $percentile25_index = round (0.25*$num_items);
	my $percentile75_index = round (0.75*$num_items);
	my $percentile5_index= round (0.05*$num_items);
	my $percentile95_index = round (0.95*$num_items);

#print "in here after median index $percentile25_index\n";
	if ($num_items % 2 == 1) {
                $median_index = floor ($num_items/2);
                return ($sorted_values[$median_index], $sorted_values[$percentile25_index], $sorted_values[$percentile75_index],   $sorted_values[$percentile5_index], $sorted_values[$percentile95_index], $sorted_values[0], $sorted_values[$#sorted_values]   );
        } else {
                $median_index = $num_items/2;
                return ( ($sorted_values[$median_index] + $sorted_values[$median_index-1]) / 2,  $sorted_values[$percentile25_index], $sorted_values[$percentile75_index], $sorted_values[$percentile5_index], $sorted_values[$percentile95_index], $sorted_values[0], $sorted_values[$#sorted_values] ); 
        }


}

sub
correlation
{
	my ($x_array_ref, $y_array_ref) = @_;
	my @x = @{$x_array_ref};
	my @y = @{$y_array_ref};

 if($#x==$#y){
 my $N = $#x;
# print "N = $N\n";
 my $mean_x = mean (@x);
 my $mean_y = mean (@y);
# print "meanx $mean_x $mean_y\n";
 my $ss_xx = 0;
 my $ss_yy = 0;
 my $ss_xy = 0;

 for(my $i=0;$i< @x;$i++){
	$ss_xx += ($x[$i] - $mean_x) **2;
	$ss_yy += ($y[$i] - $mean_y) **2;
	$ss_xy += ($x[$i] - $mean_x) * ($y[$i] - $mean_y);
 }
 my $pop_sd_x = sqrt( $ss_xx );
 my $pop_sd_y = sqrt( $ss_yy );
 my $cov_x_y = $ss_xy;
 my $correlation = $cov_x_y / ($pop_sd_x * $pop_sd_y);
	return ($correlation);
 } # check x == y
 else { return -100;}

}

sub
chisquare_2_2
{
	my (@rows) = @_;

my @row_total;
for my $i (0..(2-1)) {
  my $t = sum_chisquare(@{ $rows[$i] });

  if (0 == $t) {
    die "Cannot calculate: row $i sums to 0\n";
  }

  push @row_total, $t;
}

# Let's sum down.
my @column_total;
for my $j (0..(2-1)) {
  my $t = sum_chisquare(
    map $_->[$j], @rows
  );

  if (0 == $t) {
    die "Cannot calculate: column $j sums to 0\n";
  }

  push @column_total, $t;
}

# And the grand total.
my $total = sum_chisquare(@row_total);

# Now we calculate our expectations and chi-square together.
my $chi_square = 0;
for my $i (0..(2-1)) {
  for my $j (0..(2-1)) {
    my $expected = $row_total[$i] * $column_total[$j] / $total;

    if ($expected < 10) {
#      print "WARNING: expected[$i][$j] = $expected < 10\n";
    }

    $chi_square += ($rows[$i][$j] - $expected)**2 / $expected;
  }
}
#print "chisqurae $chi_square\n";
return ( chisqrprob( (2-1) * (2-1), $chi_square) , $chi_square ); 

}

sub sum_chisquare {
  my $total = shift;
  $total += $_ for @_;
  return $total; 
}

sub
RR
{
        my ($a, $b, $c, $d) = @_;

        if ($a == 0 || $b == 0 || $c == 0 || $d == 0) {
                return (0,0,0);
        }
        my $var_log = 1/$a - 1/($a + $b) + 1/$c - 1/($c+$d);
        my $rr = ($a * $d) / ($b * $c);

        my $rr_sqrt = sqrt ($var_log);
        my $low = exp (log ($rr) - 1.96 * $rr_sqrt);
        my $high = exp (log($rr) + 1.96*  $rr_sqrt);
#print "rr $rr low $low\n";

        return ($rr, $low, $high);
}

sub
OR
{
	my ($a, $b, $c, $d) = @_;

	if ($a == 0 || $b == 0 || $c == 0 || $d == 0) {
                return (0,0,0);
        }

	my $OR = $a * $d / ($c * $b); 
	my $se = sqrt (1/$a + 1/$b + 1/$c + 1/$d);
	my $log_OR = log ($OR);
	my $low = exp ($log_OR - 1.96* $se);
	my $high = exp ($log_OR + 1.96 * $se);
	return ($OR, $low, $high);

}

 1;


